﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Runtime.Serialization;
using System.Linq;

using HarmonyLib;

using Newtonsoft.Json;

using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;

namespace Automap
{
	public delegate void BlockDesignatorAction(ICoreClientAPI clientAPI, PointsOfInterest poi, BlockPos posn, Block block);

	/// <summary>
	/// Point of Interest Rule Designator
	/// </summary>
	[JsonObject(MemberSerialization.OptIn)]
	public class BlockDesignator
	{
		[JsonProperty]
		public Color OverwriteColor;

		[JsonIgnore]
		public BlockDesignatorAction SpecialAction;

		[JsonProperty]
		public string SpecialActionName;

		[JsonProperty]
		public AssetLocation Pattern;

		[JsonProperty]
		public EnumBlockMaterial? Material;

		[JsonProperty]
		public bool Enabled { get; set; }

		private BlockDesignator()
		{
			//throw new NotSupportedException();
		}

		public BlockDesignator(AssetLocation pattern, Color overwriteColor, EnumBlockMaterial? material)
		{
			this.Pattern = pattern;
			this.OverwriteColor = overwriteColor;
			this.Material = material;
			this.Enabled = true;
		}

		public BlockDesignator(AssetLocation pattern, Color overwriteColor, EnumBlockMaterial? material, BlockDesignatorAction specialAct)
		{
			this.Pattern = pattern;
			this.OverwriteColor = overwriteColor;
			this.Material = material;
			this.SpecialAction = specialAct;
			this.SpecialActionName = specialAct.Method.Name;
			this.Enabled = true;
		}

		public BlockDesignator(AssetLocation pattern, Color overwriteColor, EnumBlockMaterial? material, BlockDesignatorAction specialAct, bool state)
		{
		this.Pattern = pattern;
		this.OverwriteColor = overwriteColor;
		this.Material = material;
		this.SpecialAction = specialAct;
		this.SpecialActionName = specialAct.Method.Name;
		this.Enabled = state;
		}

		public override string ToString()
		{
			return $"{Pattern.ToShortString()}  | {OverwriteColor.Name} | {(Material.HasValue ? Material.ToString() : "?")} | [{SpecialActionName}]";
		}

		[OnDeserialized]
		public void RelinkDesignator(StreamingContext sCtx )
		{			
			if (SpecialAction == null && !String.IsNullOrEmpty(SpecialActionName)) {
			//TODO: still not truely dynamic...			

			BlockDesignator delegateTarget = DefaultDesignators.DefaultBlockDesignators.FirstOrDefault((BlockDesignator arg) => arg.SpecialActionName == SpecialActionName);          	
			
			if (delegateTarget != null)	this.SpecialAction = delegateTarget?.SpecialAction;

			}
		}
	}
}

