﻿using System;
using System.Linq;

using Vintagestory.API.MathTools;

namespace Automap
{
	public struct ColumnCounter
	{
		private bool NewOrLoaded;
		private byte[] EditTally;
		private byte HeightCutoff; //CHUNK #

		public uint WeightedSum 
		{
			get
			{//TODO: Rank ZERO for deep sub-surface chunks
				return ( uint )(EditTally.Skip(HeightCutoff - 1).Sum(ed => ed)) + (NewOrLoaded ? 100u : 0u);
			}
		}

		public ColumnCounter(int chunkSize)
		{
		NewOrLoaded = false;
		EditTally = new byte[chunkSize];
		HeightCutoff = 0;
		}

		public ColumnCounter(int chunkSize, bool editLoaded)
		{
		NewOrLoaded = editLoaded;
		EditTally = new byte[chunkSize];
		HeightCutoff = 0;
		}

		public ColumnCounter(int chunkSize, bool editLoaded, Vec3i chunkCoord)
		{
		int chunkY = chunkCoord.Y % chunkSize;

		NewOrLoaded = editLoaded;
		EditTally = new byte[chunkSize];

		EditTally[chunkY]++;
		HeightCutoff = ( byte )chunkY;
		}

		public ColumnCounter Update(Vec3i chunkCoord, int chunkSize, bool partlyNewOrLoaded = false)
		{
		int chunkY = chunkCoord.Y % chunkSize;
		if (EditTally[chunkY] < Byte.MaxValue) EditTally[chunkY]++;
		if (partlyNewOrLoaded) NewOrLoaded = true;

		return this;
		}

		public void SetCutoff(int height)
		{

		HeightCutoff = ( byte )height;	
		
		}

		public override string ToString( )
		{
			return $"{(NewOrLoaded?"NEW":"Old")} W:{WeightedSum}";
		}
}
}

